
package gpsys.cos;

import gpsys.*;
import java.io.*;
import java.awt.*;
import java.awt.Graphics.*;

/**
 * The user interface for the terrain robot problem.  This class
 * also contains the main() function to execute the application.
 *
 * @version		1.1, 24 Mars 1999
 * @author 	<a href="mailto:epierre@e-nef.com">Emmanuel PIERRE</a>
 *			<address>DEA IARFA,</address>
 */
public class CosSim extends Frame implements GPObserver {
	Terrain terrain;
	/**
	 * The file prefix to use for saving generations and writing reports.
	 */
	String filePrefix;

        Panel panel1;
        Panel panel2;
        Panel panel3;

	/**
	 * Construct a terrain robot user interface.
	 *
	 * @param terrain	The terrain being mowed.
	 * @param terrain	The file prefix to use for saving generations and writing
	 *				reports.
	 */
	public CosSim(Terrain terrain, String filePrefix) {
		super("Cos Sim");				// sets the title of the frame

		this.filePrefix = filePrefix;
		this.terrain = terrain;

		// a terrain viewer allows us to visualise the state of the terrain.
		TerrainViewer terrainViewer = new TerrainViewer(terrain);
		terrain.setObserver(terrainViewer);

		setSize(510,670);
		setLayout(new BorderLayout());

		panel1 = new Panel();
		panel1.resize(510,30);
		panel2 = new Panel();
		panel2.resize(510,590);
		panel3 = new Panel();
		panel3.resize(510,10);

		add("North",panel1);	
		add("Center",panel2);	
		add("South",panel3);	

		panel1.setLayout(new FlowLayout());
		panel1.add(terrainViewer.txtA);

		panel2.setLayout(new GridLayout());
                panel2.add(terrainViewer);

		panel3.setLayout(new FlowLayout());
		panel3.add(terrainViewer.txt);

		terrainViewer.txt.setText("test !\n");


	}

	/**
	 * If the filePrefix is null, just write a report of the current generation
	 * to the standard output.  Otherwise, also append the report to the file
	 * "filePrefix.txt" and save the current generation to the file
	 * "filePrefix.p1.gzip".  If the termination criteria has been met, or the
	 * maximum number of generations have been reached, visualise the
	 * best terrain robot in action!!!
	 *
	 * @param gpParameters	The GP parameters used for this run.
	 * @param how			How the generation was created.  Can be either
	 *                      CREATION, FROMSTREAM or EVOLVED.
	 */
	public void generationUpdate(GPParameters gpParameters, int how) {

		// if a file prefix was given, write the report to file and save the
		// current generation.  This need not be done if the generation was
		// just loaded from file.
		if ((filePrefix != null) &&
			(how == GPObserver.CREATION || how == GPObserver.EVOLVED)) {
			try {
				diagnosticUpdate("Saving current generation...");
				gpParameters.save(filePrefix);
				diagnosticUpdate("Saved  current generation.");
			}
			catch (IOException e) {
				System.out.println("gpParameters.save() : " + e);
				System.exit(1);
			}

			try {
				gpParameters.writeReport(filePrefix,how == GPObserver.CREATION);
			}
			catch (IOException e) {
				System.out.println("gpParameters.writeReport() : " + e);
				System.exit(1);
			}
		}

		// now write a report of the current population to the standard output
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		gpParameters.writeReport(pw,
			(how == GPObserver.CREATION) || (how == GPObserver.FROMSTREAM));
		System.out.print(sw.toString());

		// if the termination criteria has been reached or the maximum number of
		// generations have been evolved, visualise the best robot in action.
		if (gpParameters.population.best.fitness.terminationCondition() ||
			(gpParameters.population.generation ==
				(gpParameters.generations - 1))) {
			show();
			terrain.enableObserver();
			// INFOS
	                terrain.observer.printInfo(gpParameters.population.best.fitness.toString());
			//+" L0:"+terrain.l0+" L1:"+terrain.l1+" L2:"+terrain.l2+"\n");

        	        terrain.observer.printTxt(gpParameters.population.best.toString()+"\n");
			gpParameters.fitness.instance(
				gpParameters, gpParameters.population.best);

			terrain.disableObserver();
			System.out.println("GP run completed..");
			System.exit(0);
		}
	}

	/**
	 * We are not interested in this update, so we just ignore it.
	 *
	 * @param	gpParameters	the GP parameters used for this run.
	 * @param	i				the Individual that has just been created.
	 * @param	creationIndex	the index of the Individual in the population.
	 * 
	 */
	public void individualUpdate(GPParameters gpParameters,
		Individual i, int creationIndex) {
	}

	/**
	 * We are not interested in this update, so we just ignore it.
	 *
	 * @param	gpParameters	the GP parameters used for this run.
	 * @param	individualIndex	the index of the created Individual in the
	 *							population.
	 * @param	creationMethod	how the Individual was created.  Can be either
	 *							VIA_MUTATION or VIA_CROSSOVER.
	 */
	public void individualUpdate(GPParameters gpParameters,
		int individualIndex, int creationMethod) {
	}

	/**
	 * Print the diagnostic message to the standard output.
	 *
	 * @param s	The diagnostic message.
	 */
	public void diagnosticUpdate(String s) {
		System.out.println(s);
	}

	/**
	 * Print the exception and generate a stack trace on the standard output.
	 *
	 * @param e	The exception that was genearted.
	 */
	public void exception(GPException e) {
		System.out.println(e.getMessage());
		e.printStackTrace();
		System.exit(0);
	}

	/**
	 * The main() method of the terrain robot application.  This
	 * application may be invoked in any of the following ways.
	 * <pre>
	 *		java CosSim &lt;filePrefix&gt;
	 *		java CosSim &lt;filePrefix&gt; &lt;generations&gt;
	 *		java CosSim &lt;filePrefix&gt; &lt;RNG seed&gt; &lt;population&gt; &lt;generations&gt;
	 *		java CosSim &lt;RNG seed&gt; &lt;population&gt; &lt;generations&gt;
	 * </pre>
	 * 
	 * The first istructs the application to restart from the last saved
	 * session using the files with the prefix specified.  The second is the
	 * same as the first, execept that the maximum number of geneartion is
	 * modified as specified.  The third istructs the application to start
	 * a new run using the specifed file prefix for saves, and the
	 * specified parameters for the run.  The last is the similar to the
	 * previous invokation except that nothing is saved to disk, and is hence
	 * very fast.
	 */
	public static void main(String[] argv) {

		// change this to whatever is suitable to you!!
		java.util.Locale.setDefault(java.util.Locale.UK);

		CosSimGPParameters gpParameters1 = null;
		String filePrefix = null;

		// now read the command line arguments
		switch(argv.length) {
			case 1: // load entirely from saved session
			case 2: // load from saved session but change the max generations
				filePrefix = argv[0];
				try {
					System.out.println("Loading last saved generation 1...");
					gpParameters1 =
						(CosSimGPParameters) GPParameters.load(filePrefix+"1");
					if (gpParameters1 != null)
						System.out.println("Loaded last saved generation 1...");
				}
				catch (java.io.IOException e) {
					System.out.println("Loading problem 1: " + e);
					System.exit(1);
				}
				catch (ClassNotFoundException e) {
					System.out.println("Loading problem 1: " + e);
					System.exit(1);
				}
				if (argv.length == 2) {
					int generations	= Integer.parseInt(argv[1]);
					gpParameters1.generations = generations;
				}
				break;
			case 3: {
					long rngSeed	= Long.parseLong(argv[0]);
					int population	= Integer.parseInt(argv[1]);
					int generations	= Integer.parseInt(argv[2]);
					gpParameters1 = new
						CosSimGPParameters(rngSeed, population, generations);
				}
				break;
			case 4: {
					filePrefix 	= argv[0];
					long rngSeed	= Long.parseLong(argv[1]);
					int population	= Integer.parseInt(argv[2]);
					int generations	= Integer.parseInt(argv[3]);
					gpParameters1 	= new
						CosSimGPParameters(rngSeed, population, generations);
				}
				break;
			default:
				System.out.println("Usage : CosSim <file>");
				System.out.println("        CosSim <file> <generations>");
				System.out.println("        CosSim " +
					"<RNG seed> <population> <generations>");
				System.out.println("        CosSim " +
					"<file> <RNG seed> <population> <generations>");
				return;
		}

		// set the observer to be an instance of our user interface
		gpParameters1.observer = new
			CosSim(gpParameters1.terrain, filePrefix+"1");

		// create a new GP system
		GPsys gpSys = new GPsys(gpParameters1);
		// and start evolving !!!
		gpSys.evolve();
	}
}
